﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=9.3
@EndOfDesignText@
' Cicular Buffer Class
' Adapted from 17 May 2017 by Phillip Johnston • Last updated 7 February 2022
' Adapted from C Code by R J Garnett Feb 2022



Sub Class_Globals
	Private buffer As List
	Private head As Int
	Private tail As Int
	Private maxSize As Int
	Private noFields As Int
	Private full As Boolean
End Sub


'============================================================================================
'Initializes the circular buffer setting the max size and the number of data elements per record
Public Sub Initialize(listSize As Int, numberOfFields As Int)
	buffer.Initialize
	head = 0
	tail = 0
	maxSize = listSize
	noFields = numberOfFields
	full = False
End Sub


'============================================================================================
' Resets buffer to empty and sets all fields and table entries to the wipValue if wipeData is true
' Otherwise the buffer contents are unchanged.
Public Sub cbuf_reset(wipeData As Boolean, wipeValue As Int)
	head = 0
	tail = 0
	full = False
	If wipeData = True Then
		For k = 0 To buffer.Size - 1
			Private  arr(noFields) As Int
			For j = 0 To noFields - 1
				arr(j) = wipeValue
			Next
			buffer.Set(k, arr)
		Next
	End If
End Sub



'============================================================================================
' Returns the current number of elements in the buffer 
Public Sub cbuf_size As Int
	If full = False Then
		If head > = tail Then
			Return head - tail
		Else
			Return maxSize + head - tail
		End If
	End If
	Return maxSize
End Sub


'============================================================================================
' Returns the maximum cpacity of data items in the buffer
public Sub cbuf_capacity As Int
	Return maxSize
End Sub


'============================================================================================
' Places a data item in the buffer and updates the head pointer
' Unread data will be overwritten if the buffer is no popped fast enough
public Sub cbuf_put(data() As Int)
	If buffer.Size < maxSize Then
		buffer.Add(data)
	Else
		buffer.Set(head, data)
	End If
	advance_head_pointer
End Sub


'============================================================================================
' If the buffer isn't full the item will be placed in the buffer and the function will return true. 
' If the buffer is full the data is discarded and the function resturns false
public Sub cbuf_try_put(data() As Int) As Boolean
	Dim r As Boolean
	r = False

	If cbuf_full == False Then
		If buffer.Size < maxSize Then
			buffer.Add(data)
		Else
			buffer.Set(head, data)
		End If
		advance_head_pointer
		r = True
	End If
	Return r
End Sub


'============================================================================================
' Gets data at the position pointed to be the tail and returns the value in data. In this 
'case the function returns True
' If the buffer has no  new entries i.e. tail == had then the data is invaid and the function
' returns false

Public Sub cbuf_pop As List
	Dim outList As List
	outList.Initialize
	
	If cbuf_empty = False Then
		outList.Add(buffer.Get(tail))
		tail = advance_headtail_value(tail)
		full = False
	End If
	Return outList
End Sub


'============================================================================================
' Helper function to manage both head and tail pointers
private Sub advance_headtail_value(value As Int) As Int
	value = value + 1
	If value = maxSize Then
		Return  0
	End If
	Return value
End Sub


'============================================================================================
' Helper function to manage both headpointer
Private Sub advance_head_pointer
	If cbuf_full Then
		tail = advance_headtail_value(tail)
	End If
	head = advance_headtail_value(head)
	If head = tail Then
		full = True
	Else
		full = False
	End If
End Sub


'============================================================================================
public Sub cbuf_full As Boolean
	Return full
End Sub


'============================================================================================
public Sub cbuf_empty As Boolean
	If Not(full)  And head = tail Then
		Return True
	End If
	Return False
End Sub


'============================================================================================
' Examine the data at a polace relative to the tail and return the value in data
' Data is not removed from buffer.
public Sub cbuf_peek(look_ahead_counter As Int) As List
	Dim pos As Int
	Dim outList As List
	outList.Initialize
	
	Dim data(noFields) As Int
		
	'We can't look beyond the current buffer size
	If  cbuf_empty Or look_ahead_counter > cbuf_size Then
		Return outList
	End If

	pos = tail
	For k = 0 To look_ahead_counter - 1
		pos = advance_headtail_value(pos)
	Next
		
	data = 	buffer.Get(pos - 1)
	outList.Add(data)
	Return outList
End Sub


'============================================================================================
' Calculates the perecentage reamaing of the buffer.
public Sub cbuf_remaining_pct As Double
	If full = False Then
		If tail - head >= 0 Then
			Return (tail - head) / maxSize * 100.0f
		Else
			Return (maxSize - head + tail) / maxSize * 100.0f
		End If
	End If
	Return 100.0
End Sub



'============================================================================================
' Calculates the actual size of the list that has been generated
Public Sub cbuf_list_size As Int
	Return buffer.Size
End Sub
